//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;
using Server.Spells;

namespace Server.Mobiles
{
	[CorpseName( "an ancient corpse" )]
	public class AncientMummy : BaseQuestMonster
	{
		[Constructable]
		public AncientMummy() : base( AIType.AI_Mage, FightMode.Closest, 0.2, 0.4 )
		{
			Name = "An Ancient Mummy";
			Body = 0x9A;
			Hue = 62;
			
			SetStr( 100 );
			SetDex( 250 );
			SetInt( 250 );

			SetHits( 15000 );
			SetMana( 25000 );
			SetStam( 25000 );

			SetDamage( 50, 75 );

			ActiveSpeed = 0.9;
			PassiveSpeed = 0.9;

			SetDamageType( ResistanceType.Physical, 50 );
			SetDamageType( ResistanceType.Poison, 50 );

			SetResistance( ResistanceType.Physical, 70);
			SetResistance( ResistanceType.Fire, 70);
			SetResistance( ResistanceType.Cold, 70);
			SetResistance( ResistanceType.Poison, 70);
			SetResistance( ResistanceType.Energy, 70);

			SetSkill( SkillName.MagicResist, 0.0, 120.0 );
			SetSkill( SkillName.Tactics, 0.0, 120.0 );
			SetSkill( SkillName.Wrestling, 0.0, 120.0 );
			SetSkill( SkillName.Poisoning, 0.0, 120.0 );
			SetSkill( SkillName.Parry, 0.0, 120.0 );
			SetSkill( SkillName.Anatomy, 0.0, 120.0 );
			SetSkill( SkillName.Magery, 120.0 );
			SetSkill( SkillName.EvalInt, 120.0 );
			SetSkill( SkillName.Meditation, 120.0);

			VirtualArmor = 80;
			
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			PackItem( new BloodyBandage1() );
			PackItem( new BloodyBandage2() );
			}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
		public override Poison PoisonImmune{ get{ return Poison.Lethal; } }
		public override Poison HitPoison{ get{ return Poison.Lethal; } }

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			double chanceofspecialmove = .5;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				if ((defender.Name == "an energy vortex" || defender.Name == "a blade spirit" ) && defender.Player == false)
					{
						this.SendMessage( "You Dispel the Energy Vortex" );
						defender.SendMessage( "You are Dispelled" );
						defender.Delete();
					}
			}
		base.OnGaveMeleeAttack( defender );
		}
		
		public override void OnGotMeleeAttack(Mobile attacker)
		{
			double chanceofspecialmove = .2;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: // Dispel
										
						if ((attacker.Name == "an energy vortex" || attacker.Name == "a blade spirit") && attacker.Player == false)
						{
							attacker.Delete();	
							this.SendMessage( "You Dispel the Energy Vortex" );
							attacker.SendMessage( "You are Dispelled" );
						}
						break;
					
					case 2: // Send Earth Ele
						if (attacker.Player == true)
						{
						SpawnEle(attacker);
						}
						break;
				}
			}
		base.OnGotMeleeAttack( attacker );
		}

	public override void OnDamagedBySpell( Mobile attacker )
		{
			double chanceofspecialmove = .2;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: // Poison Blast
										
						if (attacker.Player == true)
						{
							attacker.ApplyPoison( attacker, HitPoison );
							this.Say( "Feel the wrath from a thousand years of Rott" );
							attacker.SendMessage( "You feel the power of the Poison Blast" );
						}
						break;
					
					case 2: // Send EV
						if (attacker.Player == true)
						{
						SpawnEV(attacker);
						}
						break;
				}
			}
		base.OnDamagedBySpell( attacker );
		}
	
	public void SpawnEV( Mobile target ) 
      { 
      this.Say("I have learnt from you in the past, take this!"); 

            EnergyVortex ev = new EnergyVortex(); 

            ev.FightMode = FightMode.Closest; 
            ev.Map = this.Map; 
            ev.Location = new Point3D( target.Location.X+1, target.Location.Y+1, target.Location.Z ); 
            ev.Combatant = target; 
      }
		
	public void SpawnEle( Mobile target ) 
      { 
      this.Say("I shall summon aid to me from the earth..."); 

            SummonedEarthElemental ele = new SummonedEarthElemental(); 

            ele.FightMode = FightMode.Closest; 
            ele.Map = this.Map; 
            ele.Location = new Point3D( target.Location.X+1, target.Location.Y+1, target.Location.Z ); 
            ele.Combatant = target; 
      }		
      
		public AncientMummy( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
