//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "an Evil corpse" )]
	public class BaronOfEvil : BaseQuestMonster
	{
		public override bool AlwaysMurderer{ get{ return true; } }
			
		[Constructable]
		public BaronOfEvil() : base( AIType.AI_Mage, FightMode.Closest, 0.15, 0.3)
		{
			Name = "A Baron of Evil";
			Body = 400;
			HueMod = 1;

			SetStr( 150 );
			SetDex( 150 );
			SetInt( 10000 );

			SetHits( 20000 );
			SetMana( 40000 );
			SetStam( 300 );

			SetDamage( 25, 40 );

			SetDamageType( ResistanceType.Physical, 20 );
			SetDamageType( ResistanceType.Poison, 20 );
			SetDamageType( ResistanceType.Energy, 20 );
			SetDamageType( ResistanceType.Fire, 20 );
			SetDamageType( ResistanceType.Cold, 20 );

			SetResistance( ResistanceType.Physical, 50);
			SetResistance( ResistanceType.Fire, 50);
			SetResistance( ResistanceType.Cold, 50);
			SetResistance( ResistanceType.Poison, 50);
			SetResistance( ResistanceType.Energy, 50);

			SetSkill( SkillName.MagicResist, 0.0, 120.0 );
			SetSkill( SkillName.Tactics, 0.0, 120.0 );
			SetSkill( SkillName.Macing, 0.0, 120.0 );
			SetSkill( SkillName.Poisoning, 0.0, 100.0 );
			SetSkill( SkillName.MagicResist, 0.0, 120.0 );
			SetSkill( SkillName.Parry, 0.0, 120.0 );
			SetSkill( SkillName.Anatomy, 0.0, 120.0 );
			SetSkill( SkillName.Magery, 120.0 );
			SetSkill( SkillName.EvalInt, 120.0 );
			SetSkill( SkillName.Meditation, 120.0);
			SetSkill( SkillName.Focus, 120.0);

			VirtualArmor = 65;

			LeatherGloves gloves = new LeatherGloves();
			gloves.Name = "Barons Gloves";
			gloves.Hue = 32;
			AddItem( gloves );

			Tunic tunic = new Tunic();
			tunic.Name = "A Barons Tunic";
			tunic.Hue = 165;
			AddItem( tunic );

			Cloak cloak = new Cloak();
			cloak.Name = "A Barons Cloak";
			cloak.Hue = 165;
			AddItem( cloak );

			Kilt kilt = new Kilt();
			kilt.Name = "A Barons Kilt";
			kilt.Hue = 165;
			AddItem( kilt );

			ThighBoots thighboots = new ThighBoots();
			thighboots.Name = "Barons Boots";
			thighboots.Hue = 1;
			AddItem( thighboots );

			WizardsHat wizardshat = new WizardsHat();
			wizardshat.Name = "A Barons Hat";
			wizardshat.Hue = 165;
			AddItem( wizardshat );
			
			OrnamentOfTheMagician bracelet = new OrnamentOfTheMagician();
			bracelet.Name = "Ultrament of the Magician";
			bracelet.Hue = 1174;
			bracelet.Attributes.CastSpeed = 5;
			bracelet.Attributes.CastRecovery = 5;
			AddItem( bracelet );
			
			BlackStaff firestaff = new BlackStaff();
			firestaff.Name = "A Fire Staff";
			firestaff.Hue = 1174;
			firestaff.LootType = LootType.Cursed;
			firestaff.WeaponAttributes.HitFireball = 75;
			firestaff.Attributes.SpellChanneling = 1;
			AddItem( firestaff );
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
		public override Poison PoisonImmune{ get{ return Poison.Lethal; } }
		public override Poison HitPoison{ get{ return Poison.Lethal; } }

		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{			
			GMKiller(defender);
			SpecialMoves(defender,.11);
			Teleport(defender,1);
			SpecialDispel(defender);
		base.OnGaveMeleeAttack( defender );
		}
		
		public override void OnGotMeleeAttack(Mobile defender)
		{
			GMKiller(defender);
			SpecialMoves(defender,.11);
			Teleport(defender,1);
			SpecialDispel(defender);
		base.OnGotMeleeAttack( defender );
		}
		
		public override void OnDamagedBySpell(Mobile defender)
		{
			GMKiller(defender);
			SpecialMoves(defender,.25);
			Teleport(defender,15);
			SpecialDispel(defender);
			OnLocationKill(defender);
		base.OnDamagedBySpell( defender );
		}
		
		public override void OnHarmfulAction( Mobile defender, bool isCriminal )
		{
			GMKiller(defender);
			SpecialMoves(defender,.11);
			Teleport(defender,5);
			SpecialDispel(defender);
			OnLocationKill(defender);
		base.OnHarmfulAction( defender, false );
		}
		
		public void OnLocationKill(Mobile defender)
		{
			int rand = Utility.Random(100);
			if (rand <= 10)
			{
				if (defender != null && defender.Location == this.Location)
			{
				defender.Hits = (defender.Hits/15);
				this.Hits = this.Hits+(defender.Hits/15);
			}
			}
			else
			{
			}	
		}
		
		public void SpecialDispel(Mobile defender)
		{
			int rand = Utility.Random(100);
			if (rand <= 5)
			{
				if (defender is EnergyVortex)
			{
					this.Hits = (this.Hits + defender.Hits);
					Effects.SendLocationParticles( EffectItem.Create( defender.Location, defender.Map, EffectItem.DefaultDuration ), 0x3728, 8, 20, 5042 );
					Effects.PlaySound( defender, defender.Map, 0x201 );

					defender.Delete();
			}		
			}
			else
			{
			}		
		}
		
		public void Teleport(Mobile defender, int teleport)
		{
		int rand = Utility.Random(100);
			if (rand <= teleport)
			{
						this.Location = defender.Location;
						Effects.SendLocationParticles( EffectItem.Create( defender.Location, defender.Map, EffectItem.DefaultDuration ), 0x376A, 9, 32, 5022 );

						defender.PlaySound( 0x1FE );
			}
			else
			{
			}
		}
		
		public void SpecialMoves(Mobile defender,double passthrough)
		{
			double chanceofspecialmove = passthrough;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 0: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;
					case 1: // Bleed Attack
						this.SendLocalizedMessage( 1060159 ); // Your target is bleeding!
						defender.SendLocalizedMessage( 1060160 ); // You are bleeding!

						defender.PlaySound( 0x133 );
						defender.FixedParticles( 0x377A, 244, 25, 9950, 31, 0, EffectLayer.Waist );

						BeginBleed( defender, this );
						break;
					case 2: //Dismount
						Dismount(defender, this);
						break;

					case 3: //Dismount
						Dismount(defender, this);
						break;

				}
			}
		}
		
		public void Dismount(Mobile defender, Mobile from)
		{
			IMount mount = defender.Mount;

			if ( mount == null )
			{
				this.SendLocalizedMessage( 1060848 ); // This attack only works on mounted targets
				return;
			}

			this.SendLocalizedMessage( 1060082 ); // The force of your attack has dislodged them from their mount!

			if ( from.Mounted )
				defender.SendLocalizedMessage( 1062315 ); // You fall off your mount!
			else
				defender.SendLocalizedMessage( 1060083 ); // You fall off of your mount and take damage!

			defender.PlaySound( 0x140 );
			defender.FixedParticles( 0x3728, 10, 15, 9955, EffectLayer.Waist );

			mount.Rider = null;

			defender.BeginAction( typeof( BaseMount ) );
			Timer.DelayCall( BlockMountDuration, new TimerStateCallback( ReleaseMountLock ), defender );

			if ( !from.Mounted )
				AOS.Damage( defender, from, Utility.RandomMinMax( 15, 25 ), 100, 0, 0, 0, 0 );
		}

		private void ReleaseMountLock( object state )
		{
			((Mobile)state).EndAction( typeof( BaseMount ) );
		}

		private static Hashtable m_Table = new Hashtable();
		private static Hashtable m_BleedTable = new Hashtable();

		public static bool IsBleeding( Mobile m )
		{
			return m_BleedTable.Contains( m );
		}

		public static void BeginBleed( Mobile m, Mobile from )
		{
			Timer t = (Timer)m_BleedTable[m];

			if ( t != null )
				t.Stop();

			t = new InternalBleedTimer( from, m );
			m_BleedTable[m] = t;

			t.Start();
		}

		public static void DoBleed( Mobile m, Mobile from, int level )
		{
			if ( m.Alive )
			{
				int damage = Utility.RandomMinMax( level, level * 2 );

				if ( !m.Player )
					damage *= 2;

				m.PlaySound( 0x133 );
				m.Damage( damage, from );

				Blood blood = new Blood();

				blood.ItemID = Utility.Random( 0x122A, 5 );

				blood.MoveToWorld( m.Location, m.Map );
			}
			else
			{
				EndBleed( m, false );
			}
		}

		public static void EndBleed( Mobile m, bool message )
		{
			Timer t = (Timer)m_BleedTable[m];

			if ( t == null )
				return;

			t.Stop();
			m_Table.Remove( m );

			m.SendLocalizedMessage( 1060167 ); // The bleeding wounds have healed, you are no longer bleeding!
		}

		private class InternalBleedTimer : Timer
		{
			private Mobile m_From;
			private Mobile m_Mobile;
			private int m_Count;

			public InternalBleedTimer( Mobile from, Mobile m ) : base( TimeSpan.FromSeconds( 2.0 ), TimeSpan.FromSeconds( 2.0 ) )
			{
				m_From = from;
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				DoBleed( m_Mobile, m_From, 5 - m_Count );

				if ( ++m_Count == 5 )
					EndBleed( m_Mobile, true );
			}
		}
		


		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}

		public BaronOfEvil( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
