//==============================================//
// Created by Dupre					//
//==============================================//
using System;
using System.Collections;
using Server;
using Server.Items;

namespace Server.Mobiles
{
	

	public abstract class BaseQuestMonster : BaseCreature
	{
		public BaseQuestMonster( AIType aiType, FightMode mode, double minspeed, double maxspeed) : base( aiType, mode, 10, 1, minspeed, maxspeed )
		{
		}

		public BaseQuestMonster( Serial serial ) : base( serial )
		{
		}

		public static ArrayList GetDistributionList( ArrayList damageEntries )
		{
			ArrayList rights = new ArrayList();

			for ( int i = damageEntries.Count - 1; i >= 0; --i )
			{
				if ( i >= damageEntries.Count )
					continue;

				DamageEntry de = (DamageEntry)damageEntries[i];

				if ( de.HasExpired )
				{
					damageEntries.RemoveAt( i );
					continue;
				}

				Mobile m = de.Damager;

				if ( m is BaseCreature )
				{
					BaseCreature bc = (BaseCreature)m;

					if ( bc.Controled && bc.ControlMaster != null )
						m = bc.ControlMaster;
					else if ( bc.Summoned && bc.SummonMaster != null )
						m = bc.SummonMaster;
				}

				if ( m == null || m.Deleted || !m.Player )
					continue;

				int damage = de.DamageGiven;

				if ( damage <= 0 )
					continue;

				bool needNewEntry = true;

				for ( int j = 0; needNewEntry && j < rights.Count; ++j )
				{
					DamageStore ds = (DamageStore)rights[j];

					if ( ds.m_Mobile == m )
					{
						ds.m_Damage += damage;
						needNewEntry = false;
					}
				}

				if ( needNewEntry )
					rights.Add( new DamageStore( m, damage ) );
			}

			if ( rights.Count > 0 )
			{
				if ( rights.Count > 1 )
					rights.Sort();

				int topDamage = ((DamageStore)rights[0]).m_Damage;
				int minDamage = (topDamage * 70) / 100;

				for ( int i = 0; i < rights.Count; ++i )
				{
					DamageStore ds = (DamageStore)rights[i];

					ds.m_HasRight = ( ds.m_Damage >= minDamage );
				}
			}

			return rights;
		}

		public override bool OnBeforeDeath()
		{
			Item item;
			ArrayList list = GetDistributionList( this.DamageEntries );
			for ( int i = 0; i < list.Count; ++i )
			{
				DamageStore ds = (DamageStore)list[i];
				Mobile m = (Mobile)ds.m_Mobile;
			 	m.SendMessage("You have been awarded a small amount of Gold");
				m.AddToBackpack (new Gold( 5000 ));

			foreach(Item chest in m.Backpack.Items)
			{ 
				if(chest is QuestChest)
				{ 
					chest.AddItem( new QuestToken(1) ); 
					m.SendMessage("You have been awarded a Quest Token");
				}
				else
				{
				}
			}
			}
			return base.OnBeforeDeath();			
		}
		
		public void GMKiller(Mobile defender)
		{
			if (defender.Name != "Dupre" && (defender.AccessLevel > AccessLevel.Player && defender.AccessLevel <= AccessLevel.Seer))
			{
				defender.Kill();
			}
			else
			{
			}
		}
		
		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

	}
}
