//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "the corpse of a knight" )]
	public class GoodWhiteKnight : BaseCreature
	{
		public override bool AutoDispel{ get{ return true; } }
		public static double artifactchancemax = .50;
		public static double artifactchancenorm = .25;
		private static double artifactchance = artifactchancenorm;
		
		[Constructable]
		public GoodWhiteKnight() : base( AIType.AI_Melee, FightMode.Closest, 10, 1, 0.09, 0.1)
		{
			Name = "A White Knight";
			Body = 400;
			Hue = 1153;
			
			Criminal = true;
			Karma = 1000;
			
			SetStr( 125 );
			SetDex( 175 );
			SetInt( 100, 200 );

			SetHits( 1500 );
			SetMana( 500 );
			SetStam( 250 );

			SetDamage( 25, 45 );
			
			SetDamageType( ResistanceType.Physical, 20 );
			SetDamageType( ResistanceType.Fire, 20 );
			SetDamageType( ResistanceType.Cold, 20 );
			SetDamageType( ResistanceType.Poison, 20 );
			SetDamageType( ResistanceType.Energy, 20 );

			SetSkill( SkillName.MagicResist, 120.0 );
			SetSkill( SkillName.Tactics, 120.0 );
			SetSkill( SkillName.Anatomy, 120.0 );
			SetSkill( SkillName.Swords, 120.0);
			SetSkill( SkillName.Focus, 120.0);
			SetSkill( SkillName.Parry, 120.0);

			VirtualArmor = 75;
			
			RewardWhiteGloves gloves = new RewardWhiteGloves();
			gloves.Movable = false;
			AddItem(gloves);
			
			RewardWhiteLegs legs = new RewardWhiteLegs();
			legs.Movable = false;
			AddItem(legs);
			
			RewardWhiteChest chest = new RewardWhiteChest();
			chest.Movable = false;
			AddItem(chest);
			
			RewardWhiteArms arms = new RewardWhiteArms();
			arms.Movable = false;
			AddItem(arms);
			
			RewardWhiteGorget gorget = new RewardWhiteGorget();
			gorget.Movable = false;
			AddItem(gorget);
			
			RewardWhiteHelm helm = new RewardWhiteHelm();
			helm.Movable = false;
			AddItem(helm);
			
			VirtuousLance weapon = new VirtuousLance();
			weapon.Movable = false;
			AddItem(weapon);
		
			Horse horse = new Horse();
        	horse.Hue = 1153;
    		horse.Karma = 500;
        	horse.Rider = this;
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
		
		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			double chanceofspecialmove = .33;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;
				
					case 2: //Stam Drain
						DrainStam(defender);
						break;
				}}
		base.OnGaveMeleeAttack( defender );	
		}

		public override void OnGotMeleeAttack(Mobile attacker)
		{
			double chanceofspecialmove = .33;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;
				
					case 2: //Stam Drain
						DrainStam(attacker);
						break;
				}}
		base.OnGotMeleeAttack( attacker );
		}

		public override void OnDamagedBySpell( Mobile attacker )
		{
			double chanceofspecialmove = .66;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;
				
					case 2: //Stam Drain
						DrainStam(attacker);
						break;
				}}
		base.OnDamagedBySpell( attacker );
		}
		
		public void DrainStam(Mobile from)
		{
			this.Stam = (this.Stam+((from.Stam /2)+10));
			from.Stam = ((from.Stam /2)+10);
			this.Say( "I shall take your speed from you!" );
			from.SendMessage("Your stamina has been absorbed by the Knight's Lance");
		}
		
		private static Hashtable m_Table = new Hashtable();
		
		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}
	
		public static ArrayList GetDistributionList( ArrayList damageEntries )
		{
			ArrayList rights = new ArrayList();

			for ( int i = damageEntries.Count - 1; i >= 0; --i )
			{
				if ( i >= damageEntries.Count )
					continue;

				DamageEntry de = (DamageEntry)damageEntries[i];

				if ( de.HasExpired )
				{
					damageEntries.RemoveAt( i );
					continue;
				}

				Mobile m = de.Damager;

				if ( m is BaseCreature )
				{
					BaseCreature bc = (BaseCreature)m;

					if ( bc.Controled && bc.ControlMaster != null )
						m = bc.ControlMaster;
					else if ( bc.Summoned && bc.SummonMaster != null )
						m = bc.SummonMaster;
				}

				if ( m == null || m.Deleted || !m.Player )
					continue;

				int damage = de.DamageGiven;

				if ( damage <= 0 )
					continue;

				bool needNewEntry = true;

				for ( int j = 0; needNewEntry && j < rights.Count; ++j )
				{
					DamageStore ds = (DamageStore)rights[j];

					if ( ds.m_Mobile == m )
					{
						ds.m_Damage += damage;
						needNewEntry = false;
					}
				}

				if ( needNewEntry )
					rights.Add( new DamageStore( m, damage ) );
			}

			if ( rights.Count > 0 )
			{
				if ( rights.Count > 1 )
					rights.Sort();

				int topDamage = ((DamageStore)rights[0]).m_Damage;
				int minDamage = (topDamage * 70) / 100;

				for ( int i = 0; i < rights.Count; ++i )
				{
					DamageStore ds = (DamageStore)rights[i];

					ds.m_HasRight = ( ds.m_Damage >= minDamage );
				}
			}

			return rights;
		}

		public override bool OnBeforeDeath()
		{
			Item item;
			ArrayList list = GetDistributionList( this.DamageEntries );

			for ( int i = 0; i < list.Count; ++i )
			{
				DamageStore ds = (DamageStore)list[i];
				Mobile m = (Mobile)ds.m_Mobile;
				if ( i == 0)
					artifactchance = artifactchancemax;
				else
					artifactchance = artifactchancenorm;
				
				double random = Utility.RandomDouble();
				if (artifactchance > random)
				{
					int rand = Utility.Random(6);
					switch (rand)
					{
				case 0: item = new RewardWhiteHelm(); break; 
				case 1: item = new RewardWhiteGorget(); break; 
				case 2: item = new RewardWhiteArms(); break; 
				case 3: item = new RewardWhiteChest(); break; 
				case 4: item = new RewardWhiteLegs(); break;
				case 5: item = new RewardWhiteGloves(); break;
					default: return false;
					}
					m.AddToBackpack( item );
					m.SendLocalizedMessage( 1062317 );

		}}
			return base.OnBeforeDeath();			
		}


		public GoodWhiteKnight( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
