//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "the corpse of a knight" )]
	public class WhiteKnightLord : BaseQuestMonster
	{
		public override bool AutoDispel{ get{ return true; } }
		
		[Constructable]
		public WhiteKnightLord() : base( AIType.AI_Melee, FightMode.Weakest, 0.075, 0.1)
		{
			Name = "A White Knight-Lord";
			Body = 400;
			Hue = 1153;

			Karma = 25000;
			
			SetStr( 200 );
			SetDex( 200 );
			SetInt( 100, 200 );

			SetHits( 30000 );
			SetMana( 500 );
			SetStam( 500 );

			SetDamage( 45, 80 );
			
			SetDamageType( ResistanceType.Physical, 20 );
			SetDamageType( ResistanceType.Fire, 20 );
			SetDamageType( ResistanceType.Cold, 20 );
			SetDamageType( ResistanceType.Poison, 20 );
			SetDamageType( ResistanceType.Energy, 20 );

			SetSkill( SkillName.MagicResist, 120.0 );
			SetSkill( SkillName.Tactics, 120.0 );
			SetSkill( SkillName.Anatomy, 120.0 );
			SetSkill( SkillName.Swords, 120.0);
			SetSkill( SkillName.Focus, 120.0);
			SetSkill( SkillName.Parry, 120.0);

			VirtualArmor = 75;
			
			RewardWhiteGloves gloves = new RewardWhiteGloves();
			gloves.Movable = false;
			AddItem(gloves);
			
			RewardWhiteLegs legs = new RewardWhiteLegs();
			legs.Movable = false;
			AddItem(legs);
			
			RewardWhiteChest chest = new RewardWhiteChest();
			chest.Movable = false;
			AddItem(chest);
			
			RewardWhiteArms arms = new RewardWhiteArms();
			arms.Movable = false;
			AddItem(arms);
			
			RewardWhiteGorget gorget = new RewardWhiteGorget();
			gorget.Movable = false;
			AddItem(gorget);
			
			RewardWhiteHelm helm = new RewardWhiteHelm();
			helm.Movable = false;
			AddItem(helm);
			
			VirtuousLance weapon = new VirtuousLance();
			weapon.Movable = false;
			AddItem(weapon);
		
			Horse horse = new Horse();
        	horse.Hue = 1153;
			horse.Karma = 500;
        	horse.Hits = 1000;
        	horse.Rider = this;
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
				
		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			GMKiller(defender);
			double chanceofspecialmove = .4;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 0: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;
				
					case 1: //Stam Drain
						DrainStam(defender);
						break;
					
					case 2:
						DrainLife();
						break;
				}}
		base.OnGaveMeleeAttack( defender );	
		}

		public override void OnGotMeleeAttack(Mobile attacker)
		{
			GMKiller(attacker);
			double chanceofspecialmove = .4;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 0: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;
				
					case 1: //Stam Drain
						DrainStam(attacker);
						break;
					
					case 2:
						DrainLife();
						break;
				}}
		base.OnGotMeleeAttack( attacker );
		}

		public override void OnDamagedBySpell( Mobile attacker )
		{
			GMKiller(attacker);
			double chanceofspecialmove = .66;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 0: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;
				
					case 1: //Stam Drain
						DrainStam(attacker);
						break;
					
					case 2:
						DrainLife();
						break;
				}}
		base.OnDamagedBySpell( attacker );
		}
		
		public void DrainLife()
		{
			ArrayList list = new ArrayList();

			foreach ( Mobile m in this.GetMobilesInRange( 5 ) )
			{
				if ( m == this || !CanBeHarmful( m ) )
					continue;

				if ( m is BaseCreature && (((BaseCreature)m).Controled || ((BaseCreature)m).Summoned || ((BaseCreature)m).Team != this.Team) )
					list.Add( m );
				else if ( m.Player )
					list.Add( m );
			}

			foreach ( Mobile m in list )
			{
				DoHarmful( m );

				m.FixedParticles( 0x374A, 10, 15, 5013, 0x496, 0, EffectLayer.Waist );
				m.PlaySound( 0x231 );

				m.SendMessage( "Your lifeforce becomes drawn to his Lance." );

				int toDrain = Utility.RandomMinMax( 5, 20 );

				Hits += toDrain;
				m.Damage( toDrain, this );
			}
		}
		
		public void DrainStam(Mobile from)
		{
			this.Stam = (this.Stam+((from.Stam /2)+10));
			from.Stam = ((from.Stam /2)+10);
			this.Say( "I shall take your speed from you!" );
			from.SendMessage("Your stamina has been absorbed by the Knight's Lance");
		}
		
		private static Hashtable m_Table = new Hashtable();
		
		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}
	
		public WhiteKnightLord( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
