//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "an Angelic corpse" )]
	public class AnAngel : BaseQuestMonster
	{
		public override bool InitialInnocent{ get{ return true; } }
		public override bool CanRummageCorpses{ get{ return true; } }

		[Constructable]
		public AnAngel() : base( AIType.AI_Mage, FightMode.Closest,0.2,0.4)
		{
			Name = "An Angel";
			Body = 174;
			Hue = 1153;

			SetStr( 125 );
			SetDex( 125 );
			SetInt( 300, 500 );

			SetHits( 25000 );
			SetMana( 25000 );
			SetStam( 25000 );

			SetDamage( 10, 15 );

			ActiveSpeed = 10;
			PassiveSpeed = 10;

			SetDamageType( ResistanceType.Physical, 20 );
			SetDamageType( ResistanceType.Energy, 20 );
			SetDamageType( ResistanceType.Cold, 60 );

			SetResistance( ResistanceType.Physical, 70);
			SetResistance( ResistanceType.Fire, 70);
			SetResistance( ResistanceType.Cold, 70);
			SetResistance( ResistanceType.Poison, 70);
			SetResistance( ResistanceType.Energy, 70);

			SetSkill( SkillName.MagicResist, 0.0, 120.0 );
			SetSkill( SkillName.Tactics, 0.0, 120.0 );
			SetSkill( SkillName.Wrestling, 0.0, 120.0 );
			SetSkill( SkillName.Anatomy, 0.0, 120.0 );
			SetSkill( SkillName.Magery, 120.0 );
			SetSkill( SkillName.EvalInt, 120.0 );
			SetSkill( SkillName.Meditation, 120.0);
			SetSkill( SkillName.Focus, 120.0);

			VirtualArmor = 75;

			Karma = 25000;

			PackGold( 30000, 60000 );
			PackWeapon( 1, 2 );
			PackWeapon( 2, 3 );
			PackWeapon( 4, 5 );
			PackWeapon( 6, 7 );
			PackWeapon( 8, 9 );
			PackWeapon( 5, 5 );
			PackWeapon( 7, 7 );
			PackWeapon( 1, 10 );
			PackWeapon( 5, 10 );
			PackWeapon( 1, 8 );
			PackWeapon( 6, 9 );
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
		public override Poison PoisonImmune{ get{ return Poison.Lethal; } }
		
		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			double chanceofspecialmove = .05;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(6);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(defender, this);
						break;

					case 3: //Life Drain
						DrainLife();
						break;

					case 4: //Spawn Pixies
						SpawnPixies( defender );
						break;

					case 5: //Spawn Lich
						SpawnLich ( defender );
						break;
				}
			}
		base.OnGaveMeleeAttack( defender );
		}

		public override void OnGotMeleeAttack(Mobile attacker)
		{
			double chanceofspecialmove = .1;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(6);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(attacker, this);
						break;

					case 3: //Life Drain
						DrainLife();
						break;

					case 4: //Spawn Pixies
						SpawnPixies( attacker );
						break;

					case 5: //Spawn Lich
						SpawnLich ( attacker );
						break;

				}
			}
		base.OnGotMeleeAttack( attacker );
		}

		public void SpawnPixies( Mobile target )
		{
			Map map = this.Map;

			if ( map == null )
				return;

			this.Say( "I shall summon aid to me" );

			int newPixies = Utility.RandomMinMax( 1, 3 );

			for ( int i = 0; i < newPixies; ++i )
			{
				Pixie pixie = new Pixie();

				pixie.Team = this.Team;
				pixie.FightMode = FightMode.Evil;
				pixie.Map = map;

				bool validLocation = false;

				for ( int j = 0; !validLocation && j < 10; ++j )
				{
					int x = X + Utility.Random( 3 ) - 1;
					int y = Y + Utility.Random( 3 ) - 1;
					int z = map.GetAverageZ( x, y );

					if ( validLocation = map.CanFit( x, y, this.Z, 16, false, false ) )
						pixie.Location = new Point3D( x, y, Z );
					else if ( validLocation = map.CanFit( x, y, z, 16, false, false ) )
						pixie.Location = new Point3D( x, y, z );
				}

				if ( !validLocation )
					pixie.Location = this.Location;

				pixie.Combatant = target;
			}
		}
		
		public void SpawnLich( Mobile target )
		{
			Map map = this.Map;

			if ( map == null )
				return;

			this.Say( "I shall summon aid to me" );

			int newGoodLich = Utility.RandomMinMax( 1, 2 );

			for ( int i = 0; i < newGoodLich; ++i )
			{
				GoodLich goodlich = new GoodLich();

				goodlich.Team = this.Team;
				goodlich.FightMode = FightMode.Evil;
				goodlich.Map = map;

				bool validLocation = false;

				for ( int j = 0; !validLocation && j < 10; ++j )
				{
					int x = X + Utility.Random( 3 ) - 1;
					int y = Y + Utility.Random( 3 ) - 1;
					int z = map.GetAverageZ( x, y );

					if ( validLocation = map.CanFit( x, y, this.Z, 16, false, false ) )
						goodlich.Location = new Point3D( x, y, Z );
					else if ( validLocation = map.CanFit( x, y, z, 16, false, false ) )
						goodlich.Location = new Point3D( x, y, z );
				}

				if ( !validLocation )
					goodlich.Location = this.Location;

				goodlich.Combatant = target;
			}
		}

		public void DrainLife()
		{
			ArrayList list = new ArrayList();

			foreach ( Mobile m in this.GetMobilesInRange( 2 ) )
			{
				if ( m == this || !CanBeHarmful( m ) )
					continue;

				if ( m is BaseCreature && (((BaseCreature)m).Controled || ((BaseCreature)m).Summoned || ((BaseCreature)m).Team != this.Team) )
					list.Add( m );
				else if ( m.Player )
					list.Add( m );
			}

			foreach ( Mobile m in list )
			{
				DoHarmful( m );

				m.FixedParticles( 0x374A, 10, 15, 5013, 0x496, 0, EffectLayer.Waist );
				m.PlaySound( 0x231 );

				m.SendMessage( "You feel the life drain out of you!" );

				int toDrain = Utility.RandomMinMax( 5, 30 );

				Hits += toDrain;
				m.Damage( toDrain, this );
			}
		}

		public void Dismount(Mobile defender, Mobile from)
		{
			IMount mount = defender.Mount;

			if ( mount == null )
			{
				this.SendLocalizedMessage( 1060848 ); // This attack only works on mounted targets
				return;
			}

			this.SendLocalizedMessage( 1060082 ); // The force of your attack has dislodged them from their mount!

			if ( from.Mounted )
				defender.SendLocalizedMessage( 1062315 ); // You fall off your mount!
			else
				defender.SendLocalizedMessage( 1060083 ); // You fall off of your mount and take damage!

			defender.PlaySound( 0x140 );
			defender.FixedParticles( 0x3728, 10, 15, 9955, EffectLayer.Waist );

			mount.Rider = null;

			defender.BeginAction( typeof( BaseMount ) );
			Timer.DelayCall( BlockMountDuration, new TimerStateCallback( ReleaseMountLock ), defender );

			if ( !from.Mounted )
				AOS.Damage( defender, from, Utility.RandomMinMax( 15, 25 ), 100, 0, 0, 0, 0 );
		}

		private void ReleaseMountLock( object state )
		{
			((Mobile)state).EndAction( typeof( BaseMount ) );
		}

		private static Hashtable m_Table = new Hashtable();
		private static Hashtable m_BleedTable = new Hashtable();


		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}

		public AnAngel( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
