//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "an Evil Lich corpse" )]
	public class LichKing : BaseQuestMonster
	{
		private Mobile m_Queen;
		private bool m_SpawnedQueen;
		public override bool AlwaysMurderer{ get{ return true; } }
		public override bool CanRummageCorpses{ get{ return true; } }

		[Constructable]
		public LichKing() : base( AIType.AI_Mage, FightMode.Evil,0.2,0.4)
		{
			Name = "A Lich King";
			Body = 24;
			Hue = 33;

			SetStr( 100 );
			SetDex( 100 );
			SetInt( 300, 500 );

			SetHits( 10000 );
			SetMana( 25000 );
			SetStam( 25000 );

			SetDamage( 7, 15 );

			ActiveSpeed = 10;
			PassiveSpeed = 10;

			SetDamageType( ResistanceType.Physical, 30 );
			SetDamageType( ResistanceType.Energy, 35 );
			SetDamageType( ResistanceType.Cold, 35 );

			SetResistance( ResistanceType.Physical, 50);
			SetResistance( ResistanceType.Fire, 50);
			SetResistance( ResistanceType.Cold, 50);
			SetResistance( ResistanceType.Poison, 50);
			SetResistance( ResistanceType.Energy, 50);

			SetSkill( SkillName.MagicResist, 120.0 );
			SetSkill( SkillName.Tactics, 0.0, 120.0 );
			SetSkill( SkillName.Wrestling, 0.0, 120.0 );
			SetSkill( SkillName.Anatomy, 0.0, 120.0 );
			SetSkill( SkillName.Magery, 120.0 );
			SetSkill( SkillName.EvalInt, 120.0 );
			SetSkill( SkillName.Meditation, 120.0);
			SetSkill( SkillName.Focus, 120.0);

			VirtualArmor = 50;

			Karma = -25000;

			PackGold( 30000, 60000 );
			PackWeapon( 1, 2 );
			PackWeapon( 2, 3 );
			PackWeapon( 4, 5 );
			PackWeapon( 6, 7 );
			PackWeapon( 8, 9 );
		
			GnarledStaff lichkingstaff = new GnarledStaff();
			lichkingstaff.Name = "Staff of the Lich King";
			lichkingstaff.Hue = 33;
			lichkingstaff.LootType = LootType.Cursed;
			lichkingstaff.Attributes.SpellChanneling = 1;
			lichkingstaff.Attributes.CastSpeed = -1;
			lichkingstaff.Attributes.CastRecovery = -3;
			lichkingstaff.Attributes.RegenMana = 4;
			lichkingstaff.Attributes.SpellDamage = 10;
			lichkingstaff.Attributes.LowerRegCost = 10;
			AddItem( lichkingstaff );
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
		public override Poison PoisonImmune{ get{ return Poison.Lethal; } }
		public override Poison HitPoison{ get{ return Poison.Lethal; } }
		
		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			double chanceofspecialmove = .05;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(defender, this);
						break;

					case 3: //Spawn Lich
						SpawnLich ( defender );
						break;
				}
			}
		base.OnGaveMeleeAttack( defender );
		}

		public override void OnGotMeleeAttack(Mobile attacker)
		{
			double chanceofspecialmove = .1;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(attacker, this);
						break;

					case 3: //Spawn Lich
						SpawnLich ( attacker );
						break;

				}
						CheckQueen();
			}
		base.OnGotMeleeAttack( attacker );
		}
		
		public void SpawnLich( Mobile target )
		{
			Map map = this.Map;

			if ( map == null )
				return;

			this.Say( "I shall summon aid to me" );

			int newGoodLich = Utility.RandomMinMax( 1, 2 );

			for ( int i = 0; i < newGoodLich; ++i )
			{
				LichLord lichlord = new LichLord();

				lichlord.Team = this.Team;
				lichlord.FightMode = FightMode.Closest;
				lichlord.Map = map;

				bool validLocation = false;

				for ( int j = 0; !validLocation && j < 10; ++j )
				{
					int x = X + Utility.Random( 3 ) - 1;
					int y = Y + Utility.Random( 3 ) - 1;
					int z = map.GetAverageZ( x, y );

					if ( validLocation = map.CanFit( x, y, this.Z, 16, false, false ) )
						lichlord.Location = new Point3D( x, y, Z );
					else if ( validLocation = map.CanFit( x, y, z, 16, false, false ) )
						lichlord.Location = new Point3D( x, y, z );
				}

				if ( !validLocation )
						lichlord.Location = this.Location;

				lichlord.Combatant = target;
			}
		}

		public void Dismount(Mobile defender, Mobile from)
		{
			IMount mount = defender.Mount;

			if ( mount == null )
			{
				this.SendLocalizedMessage( 1060848 ); // This attack only works on mounted targets
				return;
			}

			this.SendLocalizedMessage( 1060082 ); // The force of your attack has dislodged them from their mount!

			if ( from.Mounted )
				defender.SendLocalizedMessage( 1062315 ); // You fall off your mount!
			else
				defender.SendLocalizedMessage( 1060083 ); // You fall off of your mount and take damage!

			defender.PlaySound( 0x140 );
			defender.FixedParticles( 0x3728, 10, 15, 9955, EffectLayer.Waist );

			mount.Rider = null;

			defender.BeginAction( typeof( BaseMount ) );
			Timer.DelayCall( BlockMountDuration, new TimerStateCallback( ReleaseMountLock ), defender );

			if ( !from.Mounted )
				AOS.Damage( defender, from, Utility.RandomMinMax( 15, 25 ), 100, 0, 0, 0, 0 );
		}

		private void ReleaseMountLock( object state )
		{
			((Mobile)state).EndAction( typeof( BaseMount ) );
		}

		private static Hashtable m_Table = new Hashtable();
		private static Hashtable m_BleedTable = new Hashtable();


		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}

		public void CheckQueen()
		{
			if ( !m_SpawnedQueen )
			{
				this.Say( "Mwhahaha!!! For I have now completed the bond with my captive" );

				m_Queen = new AnAngel();

				((BaseCreature)m_Queen).Team = this.Team;

				m_Queen.Map = this.Map;
				m_Queen.Location = this.Location;

				m_SpawnedQueen = true;
			}
			else if ( m_Queen != null && m_Queen.Deleted )
			{
				m_Queen = null;
			}
		}

		public override void AlterDamageScalarFrom( Mobile caster, ref double scalar )
		{
			CheckQueen();

			if ( m_Queen != null )
			{
				scalar *= 0.1;
			}
		}


		public LichKing( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
