//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "a Dead Tree" )]
	public class AncientTreeSpirit : BaseQuestMonster
	{
		public override bool AutoDispel{ get{ return true; } }
		private Mobile m_Queen;
		private bool m_SpawnedQueen;
		
		[Constructable]
		public AncientTreeSpirit() : base( AIType.AI_Mage, FightMode.Closest,0.2,0.4)
		{
			Name = "An Ancient Tree Spirit";
			Body = 0x12D;
			Hue = 1152;

			SetStr( 150 );
			SetDex( 100, 150);
			SetInt( 100, 200 );

			SetHits( 25000 );
			SetMana( 25000 );
			SetStam( 25000 );

			SetDamage( 15, 25 );

			SetDamageType( ResistanceType.Physical, 50 );
			SetDamageType( ResistanceType.Cold, 50 );

			SetResistance( ResistanceType.Physical, 70);
			SetResistance( ResistanceType.Fire, 70);
			SetResistance( ResistanceType.Cold, 70);
			SetResistance( ResistanceType.Poison, 70);
			SetResistance( ResistanceType.Energy, 70);

			SetSkill( SkillName.MagicResist, 0.0, 120.0 );
			SetSkill( SkillName.Tactics, 0.0, 120.0 );
			SetSkill( SkillName.Wrestling, 0.0, 120.0 );
			SetSkill( SkillName.Anatomy, 0.0, 120.0 );
			SetSkill( SkillName.Magery, 120.0 );
			SetSkill( SkillName.EvalInt, 120.0 );
			SetSkill( SkillName.Meditation, 120.0);
			SetSkill( SkillName.Focus, 120.0);

			VirtualArmor = 50;
			
			AddItem (new TreeReward1());
			AddItem (new TreeReward2());
			AddItem (new TreeReward3());
			AddItem (new TreeReward4());
			
			AddItem(new BlueLogSE());
			AddItem(new BlueLogSW());
			AddItem(new BlueLogsSE());
			AddItem(new BlueLogsSW());
			AddItem(new BlueLogsMany1());
			AddItem(new BlueLogsMany2());
			AddItem(new BlueLogSE());
			AddItem(new BlueLogSW());
			AddItem(new BlueLogsSE());
			AddItem(new BlueLogsSW());
			AddItem(new BlueLogsMany1());
			AddItem(new BlueLogsMany2());
			AddItem(new BlueLogSE());
			AddItem(new BlueLogSW());
			AddItem(new BlueLogsSE());
			AddItem(new BlueLogsSW());
			AddItem(new BlueLogsMany1());
			AddItem(new BlueLogsMany2());
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
				
		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			if (defender.FindItemOnLayer(Layer.MiddleTorso) != null && defender.FindItemOnLayer(Layer.MiddleTorso).Name == "Sash of Evil")
			{
			double chanceofspecialmove = .1;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(defender, this);
						break;
					
					case 3: //Hide
						this.Hidden = true;
						break;
				}
			}
		}
		else
		{
		double chanceofspecialmove = .4;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(defender, this);
						break;
					
					case 3: //Hide
						this.Hidden = true;
						break;
				}
			}	
		}
		CheckQueen();
		base.OnGaveMeleeAttack( defender );
		}

		public override void OnGotMeleeAttack(Mobile attacker)
		{
			if (attacker.FindItemOnLayer(Layer.MiddleTorso) != null && attacker.FindItemOnLayer(Layer.MiddleTorso).Name == "Sash of Evil")
			{
			double chanceofspecialmove = .1;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random (4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(attacker, this);
						break;
					
					case 3: //Hide
						this.Hidden = true;
						break;
				}
			}
		}
		else
		{
		double chanceofspecialmove = .4;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random (4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Dismount
						Dismount(attacker, this);
						break;
					
					case 3: //Hide
						this.Hidden = true;
						break;
				}
			}	
		}
		CheckQueen();
		base.OnGotMeleeAttack( attacker );
		}
		
		public override void OnDamagedBySpell( Mobile attacker )
		{
			if (attacker.FindItemOnLayer(Layer.MiddleTorso) != null && attacker.FindItemOnLayer(Layer.MiddleTorso).Name == "Sash of Evil")
			{
			double chanceofspecialmove = .15;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: //Life Drain
						DrainLife();
						break;
					
					case 2: //Mana Drain
						attacker.FixedParticles( 0x376A, 9, 32, 5030, EffectLayer.Waist );
						DrainMana(attacker);
						break;
					
					case 3: //Hide
						this.Hidden = true;
						break;
				}
			}
		}
		else
		{
		double chanceofspecialmove = .5;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: //Life Drain
						DrainLife();
						break;
					
					case 2: //Mana Drain
						attacker.FixedParticles( 0x376A, 9, 32, 5030, EffectLayer.Waist );
						DrainMana(attacker);
						break;
					
					case 3: //Hide
						this.Hidden = true;
						break;
				}
			}	
		}
		CheckQueen();
		base.OnDamagedBySpell( attacker );
		}
		
		public override void AggressiveAction( Mobile aggressor, bool criminal )
		{
			base.AggressiveAction( aggressor, criminal );

			Item item = aggressor.FindItemOnLayer( Layer.MiddleTorso );

			if ( item != null && item.Name == "Sash of Virtue" )
			{
				AOS.Damage( aggressor, 50, 0, 100, 0, 0, 0 );
				item.Delete();
				aggressor.FixedParticles( 0x36BD, 20, 10, 5044, EffectLayer.Head );
				aggressor.PlaySound( 0x307 );
				aggressor.SendMessage("You have betrayed the order of Virtue");
			}
		}
		
		public void DrainMana(Mobile from)
		{
			this.Mana = (this.Mana+((from.Mana / 3)+15));
			from.Mana = ((from.Mana / 3)+15);
			this.Say( "I shall have your mana for my own!" );
			from.SendMessage("Your Mana has been partially absorbed by the creature");
		}
		
		public void DrainLife()
		{
			ArrayList list = new ArrayList();

			foreach ( Mobile m in this.GetMobilesInRange( 15 ) )
			{
				if ( m == this || !CanBeHarmful( m ) )
					continue;

				if ( m is BaseCreature && (((BaseCreature)m).Controled || ((BaseCreature)m).Summoned || ((BaseCreature)m).Team != this.Team) )
					list.Add( m );
				else if ( m.Player )
					list.Add( m );
			}

			foreach ( Mobile m in list )
			{
				DoHarmful( m );

				m.FixedParticles( 0x374A, 10, 15, 5013, 0x496, 0, EffectLayer.Waist );
				m.PlaySound( 0x231 );

				m.SendMessage( "You feel the life drain out of you!" );

				int toDrain = Utility.RandomMinMax( 5, 30 );

				Hits += toDrain;
				m.Damage( toDrain, this );
			}
		}
		
		public void Dismount(Mobile defender, Mobile from)
		{
			IMount mount = defender.Mount;

			if ( mount == null )
			{
				this.SendLocalizedMessage( 1060848 ); // This attack only works on mounted targets
				return;
			}

			this.SendLocalizedMessage( 1060082 ); // The force of your attack has dislodged them from their mount!

			if ( from.Mounted )
				defender.SendLocalizedMessage( 1062315 ); // You fall off your mount!
			else
				defender.SendLocalizedMessage( 1060083 ); // You fall off of your mount and take damage!

			defender.PlaySound( 0x140 );
			defender.FixedParticles( 0x3728, 10, 15, 9955, EffectLayer.Waist );

			mount.Rider = null;

			defender.BeginAction( typeof( BaseMount ) );
			Timer.DelayCall( BlockMountDuration, new TimerStateCallback( ReleaseMountLock ), defender );

			if ( !from.Mounted )
				AOS.Damage( defender, from, Utility.RandomMinMax( 15, 25 ), 100, 0, 0, 0, 0 );
		}

		private void ReleaseMountLock( object state )
		{
			((Mobile)state).EndAction( typeof( BaseMount ) );
		}

		private static Hashtable m_Table = new Hashtable();
		private static Hashtable m_BleedTable = new Hashtable();


		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}

		public void CheckQueen()
		{
			if ( !m_SpawnedQueen )
			{
				this.Say( "I am of the Ancient Realm, fear my Guardian" );

				m_Queen = new AncientTreeGuardian();

				((BaseCreature)m_Queen).Team = this.Team;

				m_Queen.Map = this.Map;
				m_Queen.Location = this.Location;

				m_SpawnedQueen = true;
			}
			else if ( m_Queen != null && m_Queen.Deleted )
			{
				m_Queen = null;
			}
		}

		public override void AlterDamageScalarFrom( Mobile caster, ref double scalar )
		{
			CheckQueen();

			if ( m_Queen != null )
			{
				scalar *= 0.01;
			}
		}


		public AncientTreeSpirit( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
