//==============================================//
// Created by Dupre								//
//==============================================//
using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using System.Collections;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "a woodcutters corpse" )]
	public class Woodcutter : BaseQuestMonster
	{
		public override bool AlwaysMurderer{ get{ return true; } }
		public override bool AutoDispel{ get{ return true; } }
		
		[Constructable]
		public Woodcutter() : base( AIType.AI_Melee, FightMode.Closest,0.2,0.4)
		{
			Name = "The Woodcutter";
			Body = 400;
			Hue = 1864;
			
			SetStr( 100 );
			SetDex( 100 );
			SetInt( 100, 200 );

			SetHits( 20000 );
			SetMana( 10000 );
			SetStam( 250 );

			SetDamage( 20, 30 );
			
			SetDamageType( ResistanceType.Physical, 25 );
			SetDamageType( ResistanceType.Fire, 75 );

			SetResistance( ResistanceType.Physical, 70);
			SetResistance( ResistanceType.Fire, 70);
			SetResistance( ResistanceType.Cold, 70);
			SetResistance( ResistanceType.Poison, 70);
			SetResistance( ResistanceType.Energy, 70);

			SetSkill( SkillName.MagicResist, 120.0 );
			SetSkill( SkillName.Tactics, 120.0 );
			SetSkill( SkillName.Anatomy, 120.0 );
			SetSkill( SkillName.Swords, 120.0);
			SetSkill( SkillName.Focus, 120.0);
			SetSkill( SkillName.Lumberjacking, 100.0);

			VirtualArmor = 75;
			
			ThighBoots boots = new ThighBoots();
			boots.Name = "Thigh Boots, Ranger Armour";
			boots.Hue = 1436;
			AddItem( boots );
			
			StrawHat hat = new StrawHat();
			hat.Name = "Straw Hat, Ranger Armour";
			hat.Hue = 1436;
			AddItem( hat );
			
			Cloak cloak = new Cloak();
			cloak.Name = "a Cloak, Ranger Armour";
			cloak.Hue = 1436;
			AddItem( cloak );
			
			AddItem(new RangerArms());
			AddItem(new RangerChest());
			AddItem(new RangerGloves());
			AddItem(new RangerGorget());
			AddItem(new RangerLegs());
		
			AddItem(new BlueLogSE());
			AddItem(new BlueLogSW());
			AddItem(new BlueLogsSE());
			AddItem(new BlueLogsSW());
			AddItem(new BlueLogsMany1());
			AddItem(new BlueLogsMany2());
			AddItem(new BlueLogSE());
			AddItem(new BlueLogSW());
			AddItem(new BlueLogsSE());
			AddItem(new BlueLogsSW());
			AddItem(new BlueLogsMany1());
			AddItem(new BlueLogsMany2());
			AddItem(new BlueLogSE());
			AddItem(new BlueLogSW());
			AddItem(new BlueLogsSE());
			AddItem(new BlueLogsSW());
			AddItem(new BlueLogsMany1());
			AddItem(new BlueLogsMany2());
			
			WoodcuttersAxe axe = new WoodcuttersAxe();
			axe.Movable = false;
			AddItem(axe);
		}

		public override bool Unprovokable{ get{ return true; } }
		public override bool Uncalmable{ get{ return true; } }
		public override Poison HitPoison{ get{ return Poison.Lethal; } }
		
		public static readonly TimeSpan BlockMountDuration = TimeSpan.FromSeconds( 3.0 ); // TODO: Taken from bola script, needs to be verified

		public override void OnGaveMeleeAttack(Mobile defender)
		{
			if (defender.FindItemOnLayer(Layer.MiddleTorso) != null && defender.FindItemOnLayer(Layer.MiddleTorso).Name == "Sash of Virtue")
			{
			double chanceofspecialmove = .1;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(5);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Life Drain
						DrainLife();
						break;
					
					case 3: //Stam Drain
						DrainStam(defender);
						break;
					
					case 4: // Poison
						if (defender.Player == true)
						{
							defender.ApplyPoison( defender, HitPoison );
							this.Say( "My axe's poison shall pierce thee!" );
							defender.SendMessage( "You feel the poisonous venom bind, as it ravages your body" );
						}
							break;
				}}
			}
			else
			{
			double chanceofspecialmove = .2;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(5);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						defender.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						defender.PlaySound( 0x1E1 );
						defender.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( defender, defender.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Life Drain
						DrainLife();
						break;
					
					case 3: //Stam Drain
						DrainStam(defender);
						break;
					
					case 4: // Poison
						if (defender.Player == true)
						{
							defender.ApplyPoison( defender, HitPoison );
							this.Say( "My axe's poison shall pierce thee!" );
							defender.SendMessage( "You feel the poisonous venom bind, as it ravages your body" );
						}
							break;
				}
			}
			}
		base.OnGaveMeleeAttack( defender );
		}

		public override void OnGotMeleeAttack(Mobile attacker)
		{
			if (attacker.FindItemOnLayer(Layer.MiddleTorso) != null && attacker.FindItemOnLayer(Layer.MiddleTorso).Name == "Sash of Virtue")
			{
			double chanceofspecialmove = .1;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Life Drain
						DrainLife();
						break;
					
					case 3: //Stam Drain
						Effects.SendLocationParticles( EffectItem.Create( attacker.Location, attacker.Map, EffectItem.DefaultDuration ), 0x3728, 8, 20, 5042 );
						DrainStam(attacker);
						break;
				}
			}
			}
			else
			{
			double chanceofspecialmove = .2;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(4);
				switch (rand)
				{
					case 1: // Mortal Strike
						this.SendLocalizedMessage( 1060086 ); // You deliver a mortal wound!
						attacker.SendLocalizedMessage( 1060087 ); // You have been mortally wounded!

						attacker.PlaySound( 0x1E1 );
						attacker.FixedParticles( 0x37B9, 244, 25, 9944, 31, 0, EffectLayer.Waist );

						BeginWound( attacker, attacker.Player ? PlayerDuration : NPCDuration );
						break;

					case 2: //Life Drain
						DrainLife();
						break;
					
					case 3: //Stam Drain
						Effects.SendLocationParticles( EffectItem.Create( attacker.Location, attacker.Map, EffectItem.DefaultDuration ), 0x3728, 8, 20, 5042 );
						DrainStam(attacker);
						break;
				}
			}
			}
		base.OnGotMeleeAttack( attacker );
		}

		public override void OnDamagedBySpell( Mobile attacker )
		{
			if (attacker.FindItemOnLayer(Layer.MiddleTorso) != null && attacker.FindItemOnLayer(Layer.MiddleTorso).Name == "Sash of Virtue")
			{
			double chanceofspecialmove = .15;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: //Life Drain
						DrainLife();
						break;
					
					case 2: // Super Kill
						this.Hits = (this.Hits+(attacker.Hits));
						attacker.Hits = (attacker.Hits/5);
						this.Say("You shall not defeat me");
						break;
				}
			}
			}
			else
			{
			double chanceofspecialmove = .3;
			double random = Utility.RandomDouble();
			if (chanceofspecialmove > random)
			{
				int rand = Utility.Random(3);
				switch (rand)
				{
					case 1: //Life Drain
						DrainLife();
						break;
					
					case 2: // Super Kill
						if (attacker.Hits > 9)
						{
						this.Hits = (this.Hits+(attacker.Hits));
						attacker.Hits = (attacker.Hits/10);
						this.Say("You shall not defeat me");
						}
						else
						{
						this.Say("You are lucky you are so weak, the gods will not let me leech your life force");
						}
						break;
				}}
			}
		base.OnDamagedBySpell( attacker );
		}
		
		public override void AggressiveAction( Mobile aggressor, bool criminal )
		{
			base.AggressiveAction( aggressor, criminal );

			Item item = aggressor.FindItemOnLayer( Layer.MiddleTorso );

			if ( item != null && item.Name == "Sash of Evil" )
			{
				AOS.Damage( aggressor, 50, 0, 100, 0, 0, 0 );
				item.Delete();
				aggressor.FixedParticles( 0x36BD, 20, 10, 5044, EffectLayer.Head );
				aggressor.PlaySound( 0x307 );
				aggressor.SendMessage("You have betrayed the order of Evil");
			}
		}
		
		public void DrainStam(Mobile from)
		{
			this.Stam = (this.Stam+((from.Stam /2)+10));
			from.Stam = ((from.Stam /2)+10);
			this.Say( "I shall take your speed from you!" );
			from.SendMessage("Your stamina has been absorbed by the woodcutter's axe");
		}

		public void DrainLife()
		{
			ArrayList list = new ArrayList();

			foreach ( Mobile m in this.GetMobilesInRange( 15 ) )
			{
				if ( m == this || !CanBeHarmful( m ) )
					continue;

				if ( m is BaseCreature && (((BaseCreature)m).Controled || ((BaseCreature)m).Summoned || ((BaseCreature)m).Team != this.Team) )
					list.Add( m );
				else if ( m.Player )
					list.Add( m );
			}

			foreach ( Mobile m in list )
			{
				DoHarmful( m );

				m.FixedParticles( 0x374A, 10, 15, 5013, 0x496, 0, EffectLayer.Waist );
				m.PlaySound( 0x231 );

				m.SendMessage( "You feel the life drain out of you!" );

				int toDrain = Utility.RandomMinMax( 5, 30 );

				Hits += toDrain;
				m.Damage( toDrain, this );
			}
		}
		
		private static Hashtable m_Table = new Hashtable();
		
		// Mortal Strike
		public static readonly TimeSpan PlayerDuration = TimeSpan.FromSeconds( 6.0 );
		public static readonly TimeSpan NPCDuration = TimeSpan.FromSeconds( 12.0 );

		public static bool IsWounded( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginWound( Mobile m, TimeSpan duration )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m, duration );
			m_Table[m] = t;

			t.Start();

			m.YellowHealthbar = true;
		}

		public static void EndWound( Mobile m )
		{
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			m_Table.Remove( m );

			m.YellowHealthbar = false;
			m.SendLocalizedMessage( 1060208 ); // You are no longer mortally wounded.
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				EndWound( m_Mobile );
			}
		}

		public Woodcutter( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}
